/*********************************************************************
* Copyright (C) 2000 by Progress Software Corporation ("PSC"),       *
* 14 Oak Park, Bedford, MA 01730, and other contributors as listed   *
* below.  All Rights Reserved.                                       *
*                                                                    *
* The Initial Developer of the Original Code is PSC.  The Original   *
* Code is Progress IDE code released to open source December 1, 2000.*
*                                                                    *
* The contents of this file are subject to the Possenet Public       *
* License Version 1.0 (the "License"); you may not use this file     *
* except in compliance with the License.  A copy of the License is   *
* available as of the date of this notice at                         *
* http://www.possenet.org/license.html                               *
*                                                                    *
* Software distributed under the License is distributed on an "AS IS"*
* basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
* should refer to the License for the specific language governing    *
* rights and limitations under the License.                          *
*                                                                    *
* Contributors: MIP Holdings (Pty) Ltd ("MIP")                       *
*               PSC                                                  *
*                                                                    *
*********************************************************************/

TRIGGER PROCEDURE FOR CREATE OF gst_extract_log .

/* Created automatically using ERwin ICF Trigger template db/af/erw/afercustrg.i
   Do not change manually. Customisations to triggers should be placed in separate
   include files pulled into the trigger. ICF auto generates write trigger custom
   include files. Delete or create customisation include files need to be created
   manually. Be sure to put the hook in ERwin directly so as not to have you changes
   overwritten.
   User defined Macro (UDP) Summary (case sensitive)
   gst_extract_log           Expands to full table name, e.g. gsm_user
   %TableFLA            Expands to table unique code, e.g. gsmus
   %TableObj            If blank or not defined expands to table_obj with no prefix (framework standard)
                        If defined, uses this field as the object field
                        If set to "none" then indicates table does not have an object field
   XYZ                  Do not define so we can compare against an empty string

   See docs for explanation of replication macros 
*/   

&SCOPED-DEFINE TRIGGER_TABLE gst_extract_log
&SCOPED-DEFINE TRIGGER_FLA gstel
&SCOPED-DEFINE TRIGGER_OBJ extract_log_obj


DEFINE BUFFER lb_table FOR gst_extract_log.      /* Used for recursive relationships */
DEFINE BUFFER lb1_table FOR gst_extract_log.     /* Used for lock upgrades */

DEFINE BUFFER o_gst_extract_log FOR gst_extract_log.

/* Standard top of CREATE trigger code */
{af/sup/aftrigtopc.i}

  



/* Generated by ICF ERwin Template */
/* gsm_user  gst_extract_log ON CHILD INSERT RESTRICT */
IF 
    ( gst_extract_log.user_obj <> 0 ) THEN
  DO:
    IF NOT(CAN-FIND(FIRST gsm_user WHERE
        gst_extract_log.user_obj = gsm_user.user_obj)) THEN
        DO:
          /* Cannot create child because parent does not exist ! */
          ASSIGN lv-error = YES lv-errgrp = "AF ":U lv-errnum = 102 lv-include = "gst_extract_log|gsm_user":U.
          RUN error-message (lv-errgrp, lv-errnum, lv-include).
        END.
    
    
  END.

/* Generated by ICF ERwin Template */
/* gst_batch_job  gst_extract_log ON CHILD INSERT SET NULL */
IF NOT(CAN-FIND(FIRST gst_batch_job WHERE
    gst_extract_log.batch_job_obj = gst_batch_job.batch_job_obj)) THEN
  DO:
    
    ASSIGN gst_extract_log.batch_job_obj = 0 .
  END.



/* Generated by ICF ERwin Template */
/* gsm_report_definition has gst_extract_log ON CHILD INSERT RESTRICT */
IF 
    ( gst_extract_log.report_definition_obj <> 0 ) THEN
  DO:
    IF NOT(CAN-FIND(FIRST gsm_report_definition WHERE
        gst_extract_log.report_definition_obj = gsm_report_definition.report_definition_obj)) THEN
        DO:
          /* Cannot create child because parent does not exist ! */
          ASSIGN lv-error = YES lv-errgrp = "AF ":U lv-errnum = 102 lv-include = "gst_extract_log|gsm_report_definition":U.
          RUN error-message (lv-errgrp, lv-errnum, lv-include).
        END.
    
    
  END.






ASSIGN gst_extract_log.{&TRIGGER_OBJ} = getNextObj() NO-ERROR.
IF ERROR-STATUS:ERROR THEN 
DO:
    ASSIGN lv-error = YES lv-errgrp = "AF ":U lv-errnum = 31 lv-include = "gst_extract_log|the specified object number.  Please ensure your database sequences have been set correctly":U.
    RUN error-message (lv-errgrp, lv-errnum, lv-include).
END.







/* Update Audit Log */
IF CAN-FIND(FIRST gsc_entity_mnemonic
            WHERE gsc_entity_mnemonic.entity_mnemonic = 'gstel':U
              AND gsc_entity_mnemonic.auditing_enabled = YES) THEN
  RUN af/app/afauditlgp.p (INPUT "CREATE":U, INPUT "gstel":U, INPUT BUFFER gst_extract_log:HANDLE, INPUT BUFFER o_gst_extract_log:HANDLE).

/* Standard bottom of CREATE trigger code */
{af/sup/aftrigendc.i}


/* Place any specific CREATE trigger customisations here */
